/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/* Anonymous function is used not to pollute environment */
(function(Nuvola){
	var ACTIVITY_INTERVAL = 15*60*1000;
	
	if (Nuvola.libVersion && Nuvola.warn)
	{
		var gst = Nuvola.libVersion("gstreamer");
		if (gst[0] == 1 && gst[1] == 0 && Nuvola.config.warnGstreamer1dotx !== false)
		{
			var show_again = Nuvola.warn("Problematic library detected: GStreamer " + Nuvola.libs.gstreamer,
			"Nuvola Player utilizes GStreamer library to provide support for HTML5 Audio technology used by some streaming services for music playback."
			+ "\n\nUnfortunately, GStreamer library available on your system (version " + Nuvola.libs.gstreamer + ") is known to have a bug causing malfunction of Google Play Music in Nuvola Player."
			+ " Hopefully, the bug will be fixed in the next version of GStreamer library. <b>Meanwhile, you can apply a workaround. It is not perfect and has some side effects, so it is not enabled by default for that reason.</b>"
			+ "\n\n<a href=\"http://nuvolaplayer.fenryxo.cz/help/issues/gstreamer_1.0.x.html\">Learn more about the issue and how to apply the workaround</a>." );
			if (show_again === false)
			{
				Nuvola.config.warnGstreamer1dotx = show_again;
				Nuvola.saveConfig();
			}
			Nuvola.checkFlash();
		}
		else
		{
			Nuvola.checkFlash() || Nuvola.checkMp3();
		}
	}
	
	var hasClass = function(elm, className)
	{
		var classes = elm.className.split(" ");
		var i = classes.length;
		while (i--)
		{
			if (classes[i] === className)
				return true;
		}
		
		return false;
	}
	
	/* Monkey patching to use new functions not available in older API */
	if (Nuvola.addStyleSheet === undefined)
	{
		/**
		 * Adds CSS style sheet
		 * 
		 * @since                 Nuvola JS API 2.3
		 * @param css [string]    CSS styles to apply
		 * @return style element that has been applied
		 */
		Nuvola.addStyleSheet = function(css)
		{
			var head = document.getElementsByTagName('head')[0];
			var style = document.createElement('style');
			style.type = 'text/css';
			if (style.styleSheet)
				style.styleSheet.cssText = css;
			else
				style.appendChild(document.createTextNode(css));
			head.appendChild(style);
			return style;
		}
		
		/**
		 * Removes CSS style sheet
		 * 
		 * @since          Nuvola JS API 2.3
		 * @param style    style element to remove
		 * @return true on success, false otherwise
		 */
		Nuvola.removeStyleSheet = function(style)
		{
			var parent = style.parentNode;
			if (parent)
			{
				parent.removeChild(style);
				return true;
			}
			return false;
		}
	}
	
	/**
	 * Creates Google Music integration binded to Nuvola JS API
	 */
	var Integration = function(){
		/* Overwrite default commnad function */
		Nuvola.onMessageReceived = Nuvola.bind(this, this.messageHandler);
		
		/* For debug output */
		this.name = "Google Music";		
		
		/* Let's run */
		this.state = Nuvola.STATE_NONE;
		this.can_thumbs_up = null;
		this.can_thumbs_down = null;
		this.can_prev = null;
		this.can_next = null;
		this.update();
		this.extraFeatures();
	};
	
	/**
	 * Updates current playback state
	 */
	Integration.prototype.update = function(){
		// Default state
		var state = Nuvola.STATE_NONE;
		var can_prev;
		var can_next;
		var can_thumbs_up;
		var can_thumbs_down;
			
		try
		{
			var album_art = document.getElementById('playingAlbumArt').src;
		}
		catch(e)
		{
			var album_art = null;
		}
		
		try
		{
			var song = document.getElementById('playerSongTitle').firstChild;
			song = song.innerText || song.textContent;
		}
		catch(e)
		{
			var song = null;
		}
		
		var artistDiv = document.getElementById('player-artist');
		try
		{
			var artist = artistDiv.firstChild;
			artist = artist.innerText || artist.textContent;
		}
		catch (e)
		{
			var artist = null;
		}
		
		try
		{
			var album = artistDiv.nextSibling.nextSibling.firstChild;
			album = album.innerText || album.textContent;
		}
		catch (e)
		{
			var album = null;
		}
		
		try
		{
			var buttons = document.querySelector("#player .player-middle");
			var play_pause = buttons.childNodes[2];
			if (play_pause.disabled === true)
				state = Nuvola.STATE_NONE;
			else if (play_pause.className == "flat-button playing")
				state = Nuvola.STATE_PLAYING;
			else
				state = Nuvola.STATE_PAUSED;
			
			if (state != Nuvola.STATE_NONE)
			{
				can_prev = buttons.childNodes[1].disabled === false;
				can_next = buttons.childNodes[3].disabled === false;
			}
			else
			{
				can_prev = can_next = false;
			}
			
		}
		catch (e)
		{
			can_prev = can_next = false;
		}
		
		try
		{
			var thumbs = this.getThumbs();
			
			if (thumbs[0].style.visibility == "hidden")
			{
				can_thumbs_down = can_thumbs_up = false;
			}
			else
			{
				can_thumbs_up = thumbs[1].className != "selected";
				can_thumbs_down = thumbs[2].className != "selected";
			}
		}
		catch (e)
		{
			can_thumbs_down = can_thumbs_up = false;
		}
		
		this.state = state;
		
		// Submit data to Nuvola backend
		Nuvola.updateSong(song, artist, album, album_art, state);
		
		if(this.can_prev !== can_prev){
			this.can_prev = can_prev;
			Nuvola.updateAction(Nuvola.ACTION_PREV_SONG, can_prev);
		}
		if(this.can_next !== can_next){
			this.can_next = can_next;
			Nuvola.updateAction(Nuvola.ACTION_NEXT_SONG, can_next);
		}
		if(this.can_thumbs_up !== can_thumbs_up){
			this.can_thumbs_up = can_thumbs_up;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_UP, can_thumbs_up);
		}
		if(this.can_thumbs_down !== can_thumbs_down){
			this.can_thumbs_down = can_thumbs_down;
			Nuvola.updateAction(Nuvola.ACTION_THUMBS_DOWN, can_thumbs_down);
		}
		
		setTimeout(Nuvola.bind(this, this.update), 500);
	}
	
	/**
	 * Command handler
	 * @param cmd command to execute
	 */
	Integration.prototype.messageHandler = function(cmd){
		/* SJBpost is an internal Google Music JavaScript function */
		var buttons = document.querySelector("#player .player-middle");
		if (buttons)
		{
			var prev_song = buttons.childNodes[1];
			var next_song = buttons.childNodes[3];
		}
		else
		{
			var prev_song = null;
			var next_song = null;
		}
			
		try{
			switch(cmd){
			case Nuvola.ACTION_PLAY:
				if(this.state != Nuvola.STATE_PLAYING) SJBpost("playPause");
				break;
			case Nuvola.ACTION_PAUSE:
				SJBpost("pauseSong");
				break;
			case Nuvola.ACTION_TOGGLE_PLAY:
				SJBpost("playPause");
				break;
			case Nuvola.ACTION_PREV_SONG:
				if (prev_song)
					Nuvola.clickOnElement(prev_song);
				break;
			case Nuvola.ACTION_NEXT_SONG:
				if (next_song)
					Nuvola.clickOnElement(next_song);
				break;
			case Nuvola.ACTION_THUMBS_UP:
				// Emulate click on thumbs up button
				Nuvola.clickOnElement(this.getThumbs()[1]);
				break;
			case Nuvola.ACTION_THUMBS_DOWN:
				// Emulate click on thumbs down button
				Nuvola.clickOnElement(this.getThumbs()[2]);
				break;
			default:
				// Other commands are not supported
				throw {"message": "Not supported."};
			}
			console.log(this.name + ": comand '" + cmd + "' executed.");
		}
		catch(e){
			// API expects exception to be a string!
			throw (this.name + ": " + e.message);
		}
	}
	
	Integration.prototype.extraFeatures = function(){
		if (Nuvola.config.googlebar === false)
		{
			var css = "div.gb_fa.gb_Fb.gb_j.gb_Eb { display: none !important; }\n";
			Nuvola.addStyleSheet(css);
		}
		
		if (Nuvola.config.simulateActivity === true)
			setTimeout(Nuvola.bind(this, this.simulateActivity), ACTIVITY_INTERVAL);
	}
	
	Integration.prototype.simulateActivity = function(){
		var elm = document.getElementById("insideBox");
		if(elm){
			console.log("Click on " + elm);
			Nuvola.clickOnElement(elm);
			setTimeout(Nuvola.bind(this, this.simulateActivity), ACTIVITY_INTERVAL);
		}
	}
	
	
	Integration.prototype.getThumbs = function()
	{
		var elm = document.getElementsByClassName("thumbs")[0];
		return [elm, elm.childNodes[0], elm.childNodes[1]];
	}
	
	/* Store reference */ 
	Nuvola.integration = new Integration(); // Singleton

// Immediately call the anonymous function with Nuvola JS API main object as an argument.
// Note that "this" is set to the Nuvola JS API main object.
})(this);

// Force open pop-up targets in the main window, so we can apply sandbox on them too.
window.open = function(uri){
	console.log("Opening a pop-up window target in the main frame: " + uri);
	window.location.href = uri;
}
