/*
 * Copyright 2011-2013 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

namespace Nuvola{

/**
 * FrameBridge provides bridge between Vala code and JavaScript environment for a given frame.
 * 
 * It provides methods to execute scripts in that frame with {@link object_this} used in context
 * of 'this'.
 */
public class FrameBridge: GLib.Object
{
	private unowned JS.Object? _object_this = null;
	[Description(nick = "Object used for context 'this'.", blurb = "JavaScript object that can be referenced via 'this' keyword.")]
	public unowned JS.Object? object_this
	{
		get
		{
			return _object_this;
		}
		
		set
		{
			unowned JS.Context ctx = JS.Context.for_frame(frame);
			if (_object_this != null)
				_object_this.unprotect(ctx);
			_object_this = value;
			if (_object_this != null)
				_object_this.protect(ctx);
		}
	}
	
	[Description(nick = "Frame id", blurb = "Identifier of this frame.")]
	public string frame_id{get; private set;}
	
	[Description(nick = "JavaScript API", blurb = "JavaScript API associated with this bridge.")]
	public JSApi js_api{get; private set;}
	
	[Description(nick = "Web frame", blurb = "Web frame associated with this bridge.")]
	public weak WebKit.WebFrame frame{get; private set;}
	
	/**
	 * Emitted when page has been loaded.
	 */ 
	public signal void load_finished();
	
	/**
	 * Creates new FrameBridge.
	 * 
	 * @param frame_id       custom identifier of the frame
	 * @param frame          frame to be used for bridge
	 * @param js_api         JavaScript API to be used for bridge
	 * @param object_this    object to be used in context of 'this'
	 */
	public FrameBridge(string frame_id, WebKit.WebFrame frame, JSApi js_api, JS.Object? object_this=null)
	{
		this.frame_id = frame_id;
		this.frame = frame;
		this.js_api = js_api;
		this.object_this = object_this;
		frame.notify["load-status"].connect(on_load_status_changed);
		#if DEBUG_MEMORY
		debug("new FrameBridge, js_api refs: %u", js_api.ref_count);
		#endif
	}
	
	#if DEBUG_MEMORY
	~FrameBridge()
	{
		debug("~FrameBridge, js_api refs: %u", js_api.ref_count);
	}
	#endif
	
	/**
	 * Returns JavaScript context for the frame.
	 * 
	 * @return    JavaScript context
	 */
	public unowned JS.Context get_context()
	{
		return JS.Context.for_frame(frame);
	}
	
	/**
	 * Executes script from file.
	 * 
	 * The script will be executed in a context returned by {@link get_context().
	 * The "this" keyword will refer to {@link object_this} if provided.
	 * 
	 * @param file    script to execute
	 * @return        return value of the script
	 * @throw         JSError on failure
	 */
	public unowned Value execute_script_from_file(File file) throws JSError{
		string code;
		try{
			code = Diorite.System.read_file(file);
		}
		catch(Error e){
			throw new JSError.READ_ERROR("Unable to read script %s: %s",
				file.get_path(), e.message);
		}
		return execute_script(code, file.get_uri(), 1);
	}
	
	/**
	 * Executes script.
	 * 
	 * The script will be executed in a context returned by {@link get_context()}.
	 * The "this" keyword will refer to {@link object_this} if provided.
	 * 
	 * @param script    script to execute
	 * @return          return value of the script
	 * @throw           JSError on failure
	 */
	public unowned Value execute_script(string script, string path = "about:blank", int line=1) throws JSError{
		unowned JS.Context ctx = get_context(); 
		JS.Value exception = null;
		unowned Value value = ctx.evaluate_script(new JS.String(script), object_this, new JS.String(path), line=0, out exception);
		if (exception != null)
			throw new JSError.EXCEPTION(JSTools.exception_to_string(ctx, exception));
		return value;
	}
	
	private void on_load_status_changed(GLib.Object o, ParamSpec p)
	{
		var frame = o as WebKit.WebFrame;
		return_if_fail(frame != null);
		if (frame.load_status == WebKit.LoadStatus.FINISHED)
			load_finished();
	}
}

} // namespace Nuvola
