/*
 * Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met: 
 * 
 * 1. Redistributions of source code must retain the above copyright notice, this
 *    list of conditions and the following disclaimer. 
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution. 
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if UNITY_QUICK_LIST

using Diorite.Logger;
using Diorite.System;
using Unity;

namespace Nuvola.Extensions.UnityQuickList{

public Nuvola.ExtensionInfo get_info(){
	return {
		/// Name of a plugin providing scrobbling to Last.fm
		_("Unity Quick List"),
		Config.VERSION,
		/// Extension descriptiom
		_("<p>This plugin provides integration with docks compliant with Unity Quick List (Unity Launcher, DockBarX).</p>"),
		"Jiří Janoušek",
		typeof(Extension),
		true
	};
}

/**
 * Manages dock item at Unity Launcher
 */
public class Extension: Nuvola.Extension{
	private weak Diorite.Application app;
	private weak Diorite.ActionGroups action_groups;
	private weak Nuvola.Player player;
	private LauncherEntry dock_item;
	private bool active = true;
	private uint update_id = 0;
	
	/**
	 * {@inheritDoc}
	 */
	public override void load(ObjectContainer objects) throws ExtensionError
	{
		// Get dependencies, throws ExtensionError
		app = objects.get<Diorite.Application>("application");
		action_groups = objects.get<Diorite.ActionGroups>("action_groups");
		player = objects.get<Player>("player");
		
		debug("Unity Launcher Entry for %s.desktop", app.desktop_entry);
		dock_item = LauncherEntry.get_for_desktop_id(app.desktop_entry + ".desktop");
		action_groups.add_action_to_group.connect_after(on_action_added_or_removed);
		action_groups.remove_action_from_group.connect(on_action_added_or_removed);
		schedule_update();
	}
	
	/**
	 * {@inheritDoc}
	 */
	public override void unload()
	{
		remove_menu();
		action_groups.add_action_to_group.disconnect(on_action_added_or_removed);
		action_groups.remove_action_from_group.disconnect(on_action_added_or_removed);
	}
	
	private void create_menu()
	{
		var menu = new Dbusmenu.Menuitem();
		foreach (var action in action_groups.get_actions_of_group(BASE_PLAYER_ACTIONS))
			menu.child_append(create_menu_item(action));
		foreach (var action in action_groups.get_actions_of_group(EXTRA_PLAYER_ACTIONS))
			menu.child_append(create_menu_item(action));
		dock_item.quicklist = menu;
	}
	
	private Dbusmenu.Menuitem create_menu_item(Diorite.Action action){
		var item = new Dbusmenu.Menuitem();
		item.property_set(Dbusmenu.MENUITEM_PROP_LABEL, action.label);
		item.property_set(Dbusmenu.MENUITEM_PROP_ICON_NAME, action.stock_id);
		item.property_set_bool(Dbusmenu.MENUITEM_PROP_VISIBLE, action.sensitive);
		var toggle = action as Diorite.ToggleAction;
		if (toggle != null)
		{
			item.property_set(Dbusmenu.MENUITEM_PROP_TOGGLE_TYPE, Dbusmenu.MENUITEM_TOGGLE_CHECK);
			item.property_set_int(Dbusmenu.MENUITEM_PROP_TOGGLE_STATE,
			toggle.active ? Dbusmenu.MENUITEM_TOGGLE_STATE_CHECKED : Dbusmenu.MENUITEM_TOGGLE_STATE_UNCHECKED);
		}
		item.item_activated.connect(() => {action.activate();});
		action.notify.connect((o, p) => {
			switch(p.name){
			case "sensitive":
				item.property_set_bool(Dbusmenu.MENUITEM_PROP_VISIBLE, action.sensitive);
				break;
			case "label":
				item.property_set(Dbusmenu.MENUITEM_PROP_LABEL, action.label);
				break;
			case "stock-id":
				item.property_set(Dbusmenu.MENUITEM_PROP_ICON_NAME, action.stock_id);
				break;
			case "active":
				var toggle_item = action as Diorite.ToggleAction;
				if (toggle_item != null)
					item.property_set_int(Dbusmenu.MENUITEM_PROP_TOGGLE_STATE,
					toggle_item.active ? Dbusmenu.MENUITEM_TOGGLE_STATE_CHECKED : Dbusmenu.MENUITEM_TOGGLE_STATE_UNCHECKED);
				break;
			}
		});
		return item;
	}
			
	private void remove_menu()
	{
		lock(this.active){
			if(!this.active) return;
			this.active = false;
			if(this.dock_item != null && this.dock_item.quicklist != null){
				this.dock_item.quicklist = null;
			}
			
		}
	}
	
	private void on_action_added_or_removed(Diorite.Action action, string group)
	{
		schedule_update();
	}
	
	private void schedule_update()
	{
		if(update_id != 0)
			Source.remove(update_id);
		update_id = Idle.add(update_menu_cb);
	}
	
	private bool update_menu_cb()
	{
		update_id = 0;
		remove_menu();
		create_menu();
		return false;
	}
}

} // namespace Nuvola.Extensions.UnityQuickList
#endif
