using Fenryxo.Test;
using Fenryxo.Logger;

class Nuvola.ServicesManagerTest: Fenryxo.Test.TestCase{
	private Fenryxo.Storage storage;
	private ServicesManager manager;
	private Fenryxo.Config.MultiTypeMap? ui_state;
	private Fenryxo.Config.MultiTypeMap? config;
	private JSApi? js_api;
	
	construct{
		add_task("read_metadata", test_read_metadata);
		add_task("get_service", test_get_service,
			{"read_metadata"});
		add_task("list_services", test_list_services,
			{"read_metadata"});
		add_task("load", test_load, {"get_service"});
		add_task("js_api", test_js_api, {"load"});
		add_task("config", test_config, {"get_service"});
		add_task("ui_state", test_ui_state, {"get_service"});
	}
	
	public override void set_up_before(){
		storage = new Storage(APPNAME + "/services");
		manager = new ServicesManager(storage);
	}
	
	public override void set_up(){
		ui_state = new Fenryxo.Config.MultiTypeHashMap();
		config = new Fenryxo.Config.MultiTypeHashMap();
		js_api = new JSApi(storage);
	}
	
	public override void tear_down(){
		ui_state = null;
		config = null;
		js_api = null;
	}
	
	public void test_read_metadata() throws TestError{
		var googleplay = storage.get_child("googleplay");
		foreach(var dir in googleplay.data_dirs){
			var service = manager.load_service_from_dir(dir);
			Assert.not_null(service, "Service not found in " + dir.get_path());
		}
		
		var dir2 = googleplay.user_data_dir;
		Assert.not_null(dir2);
		
		var service = manager.load_service_from_dir(dir2);
		Assert.not_null(service, "Service not found in " + dir2.get_path());
		
		Assert.cmp_str(service.id, "==", "googleplay");
		Assert.cmp_str(service.name, "==", "Google Play (data_user)");
		Assert.cmp_int(service.version, "==", 80);
		Assert.cmp_str(service.maintainer_name, "==", "Jiří Janoušek");
		Assert.cmp_str(service.maintainer_link, "==", "https://launchpad.net/~fenryxo");
	}
	
	public void test_get_service() throws TestError{
		var service = manager.get_service("grooveshark");
		Assert.not_null(service, "Grooveshark service is null");
		Assert.cmp_int(service.version, "==", 1);
		
		service = manager.get_service("googleplay");
		Assert.not_null(service, "Googgle Play service is null");
		Assert.cmp_int(service.version, "==", 100);
		
		service = manager.get_service("banana");
		Assert.not_null(service, "Banana service is null");
		Assert.cmp_int(service.version, "==", 81);
		
		service = manager.get_service("apple");
		Assert.is_null(service, "I don't want apples");
		
		var name = "broken1";
		service = manager.get_service(name);
		Assert.is_null(service, name + "is not null");
		
		name = "broken2";
		service = manager.get_service(name);
		Assert.is_null(service, name + "is not null");
		
		name = "broken3";
		service = manager.get_service(name);
		Assert.is_null(service, name + "is not null");
	}
	
	public void test_list_services() throws TestError{
		var services = manager.list_services();
		var keys = services.get_keys();
		keys.sort(strcmp);
				
		Assert.cmp_uint(7, "==", keys.length());
		
		unowned List<string> pos = keys.first();
		var id = pos.data;
		Assert.cmp_str(pos.data, "==", "banana");
		var metadata = services.lookup(id);
		Assert.is_true(metadata != null, @"Metadata for id $id is null");
		Assert.cmp_int(metadata.version, "==", 81);
		Assert.cmp_str(metadata.description, "==", "<b>Banana (data_first)</b>\n");
		
//~ 		Assert.cmp_str(metadata.dir.get_path(), "~=", "tests/dirs/data_first/nuvolaplayer/services/banana");
		
		pos = pos.next;
		id = pos.data;
		Assert.cmp_str(id, "==", "eighttracks");
		metadata = services.lookup(id);
		Assert.is_true(metadata != null, @"Metadata for id $id is null");
		Assert.cmp_int(metadata.version, "==", 1);
		
		pos = pos.next;
		id = pos.data;
		Assert.cmp_str(id, "==", "googleplay");
		metadata = services.lookup(id);
		Assert.is_true(metadata != null, @"Metadata for id $id is null");
		Assert.cmp_int(metadata.version, "==", 100);
		
		pos = pos.next;
		id = pos.data;
		Assert.cmp_str(id, "==", "grooveshark");
		metadata = services.lookup(id);
		Assert.is_true(metadata != null, @"Metadata for id $id is null");
		Assert.cmp_int(metadata.version, "==", 1);
		
		pos = pos.next;
		id = pos.data;
		Assert.cmp_str(id, "==", "hypem");
		metadata = services.lookup(id);
		Assert.is_true(metadata != null, @"Metadata for id $id is null");
		Assert.cmp_int(metadata.version, "==", 1);
		
		pos = pos.next;
		id = pos.data;
		Assert.cmp_str(id, "==", "pandora");
		metadata = services.lookup(id);
		Assert.is_true(metadata != null, @"Metadata for id $id is null");
		Assert.cmp_int(metadata.version, "==", 1);
		
	}
	
	private Gtk.Window create_window_with(Gtk.Widget widget){
		var scroll = new Gtk.ScrolledWindow(null, null);
		scroll.add(widget);
		var win = new Gtk.Window();
		win.add(scroll);
		return win;
	}
	
	private AsyncContext pack_and_get_context(Gtk.Widget widget){
		var win = create_window_with(widget);
		var ctx = new AsyncContext();
		win.destroy.connect(ctx.interrupt);
		win.show_all();
		return ctx;
	}
	
	public void test_load() throws TestError{
		var view = new MainView(null);
		var ctx = pack_and_get_context(view);
		
		var service = manager.get_service("banana");
		
		Assert.cmp_str(null, "==", view.uri);
		
		ctx.handler_id = view.notify["uri"].connect(() => {
			if(view.uri != null){
				ctx.done();
			}
		});
		service.load(view, config, ui_state, js_api);
		ctx.assert_timeout(5, "Uri has not changed");
		view.disconnect(ctx.handler_id);
		service.unload();
	}
	
	public void test_js_api() throws TestError{
		var view = new MainView(null);
		var ctx = pack_and_get_context(view);
		var service = manager.get_service("banana");
		
		ctx.handler_id = service.data_changed.connect( (song, artist, album, art_uri, state, prev,
			next, up, down, fav) => {
			try{
				Assert.cmp_str(song, "==", null);
				Assert.cmp_str(artist, "==", null);
				Assert.cmp_str(album, "==", null);
				Assert.cmp_str(art_uri, "==", null);
				Assert.cmp_str(state, "==", STATE_NONE);
				Assert.is_false(prev, "Can prev is not false");
				Assert.is_false(next, "Can next is not false");
				Assert.is_false(up, "Thumbs up is not false");
				Assert.is_false(down, "Thumbs down is not false");
				Assert.is_false(fav, "Favorite is not false");
				ctx.done();
			}
			catch(TestError e){
				ctx.fail(e);
			}
		});
		service.load(view, config, ui_state, js_api);
		ctx.assert_timeout(5, "Service load failed.");
		service.disconnect(ctx.handler_id);
		
		ctx.handler_id = service.data_changed.connect( (song, artist, album, art_uri, state, prev,
			next, up, down, fav) => {
			try{
				Assert.cmp_str(song, "==", "I love Vala");
				Assert.cmp_str(artist, "==", "Jiří Janoušek");
				Assert.cmp_str(album, "==", "Best of");
				Assert.cmp_str(art_uri, "==", null);
				Assert.cmp_str(state, "==", STATE_PLAYING);
				Assert.is_false(prev, "Can prev is not false");
				Assert.is_true(next, "Can next is not true");
				Assert.is_false(up, "Thumbs up is not false");
				Assert.is_false(down, "Thumbs down is not false");
				Assert.is_false(fav, "Favorite is not false");
				ctx.done();
			}
			catch(TestError e){
				ctx.fail(e);
			}
		});
		try{
			service.command(PlayerActions.PLAY);
		}
		catch(ServiceError e){
			Assert.not_reached("JSError: " + e.message);
		}
		ctx.assert_timeout(5, "PlayerActions.PLAY failed");
		service.disconnect(ctx.handler_id);
		
		ctx.handler_id = service.data_changed.connect( (song, artist, album, art_uri, state, prev,
			next, up, down, fav) => {
			try{
				Assert.cmp_str(song, "==", "I love Vala");
				Assert.cmp_str(artist, "==", "Jiří Janoušek");
				Assert.cmp_str(album, "==", "Best of");
				Assert.cmp_str(art_uri, "==", null);
				Assert.cmp_str(state, "==", STATE_PAUSED);
				Assert.is_false(prev, "Can prev is not false");
				Assert.is_true(next, "Can next is not true");
				Assert.is_false(up, "Thumbs up is not false");
				Assert.is_false(down, "Thumbs down is not false");
				Assert.is_false(fav, "Favorite is not false");
				ctx.done();
			}
			catch(TestError e){
				ctx.fail(e);
			}
		});
		try{
			service.command(PlayerActions.PAUSE);
		}
		catch(ServiceError e){
			Assert.not_reached("JSError: " + e.message);
		}
		ctx.assert_timeout(5, "PlayerActions.PAUSE failed.");
		service.disconnect(ctx.handler_id);
		
		ctx.handler_id = service.data_changed.connect( (song, artist, album, art_uri, state, prev,
			next, up, down, fav) => {
			try{
				Assert.cmp_str(song, "==", "C is so ugly");
				Assert.cmp_str(artist, "==", "Jiří Janoušek");
				Assert.cmp_str(album, "==", "Best of");
				Assert.cmp_str(art_uri, "==", null);
				Assert.cmp_str(state, "==", STATE_PAUSED);
				Assert.is_true(prev, "Can prev is not true");
				Assert.is_true(next, "Can next is not true");
				Assert.is_false(up, "Thumbs up is not false");
				Assert.is_false(down, "Thumbs down is not false");
				Assert.is_false(fav, "Favorite is not false");
				ctx.done();
			}
			catch(TestError e){
				ctx.fail(e);
			}
		});
		try{
			service.command(PlayerActions.NEXT_SONG);
		}
		catch(ServiceError e){
			Assert.not_reached("JSError: " + e.message);
		}
		ctx.assert_timeout(5, "PlayerActions.NEXT_SONG failed.");
		service.disconnect(ctx.handler_id);
		
		ctx.handler_id = service.data_changed.connect( (song, artist, album, art_uri, state, prev,
			next, up, down, fav) => {
			try{
				Assert.cmp_str(song, "==", "C# is better");
				Assert.cmp_str(artist, "==", "Microsoft");
				Assert.cmp_str(album, "==", "Service Pack 99");
				Assert.cmp_str(art_uri, "==", null);
				Assert.cmp_str(state, "==", STATE_PAUSED);
				Assert.is_true(prev, "Can prev is not true");
				Assert.is_false(next, "Can next is not false");
				Assert.is_false(up, "Thumbs up is not false");
				Assert.is_false(down, "Thumbs down is not false");
				Assert.is_false(fav, "Favorite is not false");
				ctx.done();
			}
			catch(TestError e){
				ctx.fail(e);
			}
		});
		try{
			service.command(PlayerActions.NEXT_SONG);
		}
		catch(ServiceError e){
			Assert.not_reached("JSError: " + e.message);
		}
		ctx.assert_timeout(5);
		service.disconnect(ctx.handler_id);
		
		ctx.handler_id = service.data_changed.connect( (song, artist, album, art_uri, state, prev,
			next, up, down, fav) => {
			try{
				Assert.cmp_str(song, "==", "C is so ugly");
				Assert.cmp_str(artist, "==", "Jiří Janoušek");
				Assert.cmp_str(album, "==", "Best of");
				Assert.cmp_str(art_uri, "==", null);
				Assert.cmp_str(state, "==", STATE_PAUSED);
				Assert.is_true(prev, "Can prev is not true");
				Assert.is_true(next, "Can next is not true");
				Assert.is_false(up, "Thumbs up is not false");
				Assert.is_false(down, "Thumbs down is not false");
				Assert.is_false(fav, "Favorite is not false");
				ctx.done();
			}
			catch(TestError e){
				ctx.fail(e);
			}
		});
		try{
			service.command(PlayerActions.PREV_SONG);
		}
		catch(ServiceError e){
			Assert.not_reached("JSError: " + e.message);
		}
		ctx.assert_timeout(5, "PlayerActions.NEXT_SONG #2 failed.");
		service.disconnect(ctx.handler_id);
		
		
//~ 		var loop = new MainLoop();
//~ 		loop.run();
		
//~ 		ctx.assert_timeout(1200);
	}
	
	public void test_config() throws TestError{
		var service = manager.get_service("banana");
		var view = new MainView(null);
		service.load(view, config, ui_state, js_api);
		
		var key = "mykey";
		var str_val = "Jiří Janoušek";
		service.service_config_set_string(key, str_val);
		Assert.cmp_str(str_val, "==", service.service_config_get_string(key));
		
		double number_val = 3.14;
		service.service_config_set_double(key, number_val);
		Assert.cmp_double(number_val, "==", service.service_config_get_double(key, 0.0));
		
		var bool_val = true;
		service.service_config_set_bool(key, bool_val);
		Assert.is_true(service.service_config_get_bool(key, false));
	}
	
	public void test_ui_state() throws TestError{
		var service = manager.get_service("banana");
		var view = new MainView(null);
		service.load(view, config, ui_state, js_api);
		
		var key = "mykey";
		var str_val = "Jiří Janoušek";
		service.service_ui_state_set_string(key, str_val);
		Assert.cmp_str(str_val, "==", service.service_ui_state_get_string(key));
		
		double number_val = 3.14;
		service.service_ui_state_set_double(key, number_val);
		Assert.cmp_double(number_val, "==", service.service_ui_state_get_double(key, 0.0));
		
		var bool_val = true;
		service.service_ui_state_set_bool(key, bool_val);
		Assert.is_true(service.service_ui_state_get_bool(key, false));
	}
}


