using FsoFramework.Test;

public class Nuvola.JSApiTest: FsoFramework.Test.TestCase{
	private const string SERVICE_ID = "test_service";
	private const string FRAME_ID = "__my_frame__";
	private JSApi? js_api = null;
	private Service? service = null;
	private WebView? web_view;
	private Gtk.Window? window;
	private string[] console_messages;
	private const string JS_TESTS = "/tests/data/js/test_";
	private int counter = 0;

	public JSApiTest(){
		base("JSApiTest");
		add_test("keys_exist", test_keys_exist);
		add_test("version_info", test_version_info);
		add_test("inject_to_frame", test_inject_to_frame);
	}
	
	private Service create_service()
	{
		var storage = get_test_storage(null, {TOP + "/data/services"});
		var manager = new ServicesManager(storage);
		var service_dir = File.new_for_path(TOP + "/tests/data/services").get_child(SERVICE_ID);
		return manager.load_service_from_dir(service_dir);
	}
	
	public override void set_up(){
		window = new Gtk.Window();
		web_view = new WebView();
		web_view.developer_mode = true;
		console_messages = {};
		web_view.console_message.connect(console_handler);
		window.add(web_view);
		window.show_all();
		service = create_service();
		var storage = get_test_storage(Config.APPNAME, {TOP + "/data", TOP + "/tests/data/js"});
		js_api = new JSApi(storage, web_view, service, null, FRAME_ID, new FormatSupport(null), null);
	}
	
	public override void tear_down(){
		web_view.console_message.disconnect(console_handler);
		web_view = null;
		window.destroy();
		window = null;
		js_api = null;
		service = null;
		purge_test_dir();
	}
	
	public void test_keys_exist(){
		js_api.load_finished.connect(run_test_keys_exist);
		assert(FsoFramework.Test.wait_for_signal(500, this, "async-test-done", () => {
		web_view.load_html_string("""<html>
<head>
</head>
<body>
<p>Text</p>
</body>
</html>""", "file:///hello.html");
		}));
		js_api.load_finished.disconnect(run_test_keys_exist);
	}
	
	public void run_test_keys_exist()
	{
		try
		{
			js_api.main_bridge.execute_script_from_file(File.new_for_path(TOP + JS_TESTS + "keys.js"));
		}
		catch(JSError e)
		{
			assert_cmpstr("", Operator.EQ, e.message);
		}
		async_test_done();
	}
	
	public void test_version_info(){
		js_api.load_finished.connect(run_test_version_info);
		assert(FsoFramework.Test.wait_for_signal(500, this, "async-test-done", () => {
		web_view.load_html_string("""<html>
<head>
</head>
<body>
<p>Text</p>
</body>
</html>""", "file:///hello.html");
		}));
		js_api.load_finished.disconnect(run_test_version_info);
	}
	
	public void run_test_version_info()
	{
		var script = """
(function(Nuvola){
var data = 
{
	"VERSION_MAJOR": %d,
	"VERSION_MINOR": %d,
	"VERSION_BUGFIX": %d,
	"VERSION_SUFFIX": "%s",
	"API_VERSION": %d,
	"API_VERSION_MINOR": %d,
	"frame_id": "%s"
};

for (var key in data)
	if (data[key] !== Nuvola[key])
		throw new Error(key + ": '" + data[key] + "' !== '" + Nuvola[key] + "'");

})(this);
""".printf(Config.VERSION_MAJOR, Config.VERSION_MINOR, Config.VERSION_BUGFIX, Config.VERSION_SUFFIX,
		JSApi.API_VERSION, JSApi.API_VERSION_MINOR, FRAME_ID);
		try
		{
			js_api.main_bridge.execute_script(script, "test_version_info.js");
		}
		catch(JSError e)
		{
			assert_cmpstr("", Operator.EQ, e.message);
		}
		async_test_done();
	}
	
	public void test_inject_to_frame()
	{
		js_api.load_finished.connect(run_test_inject_to_frame);
		js_api.api_injected.connect(run_test_inject_to_frame_on_api_injected);
		counter = 6;
		js_api.song_changed.connect(on_song_changed);
		assert(FsoFramework.Test.wait_for_signal(1000, this, "async-test-done", () => {
			web_view.load_uri(service.resources.get_child("inject_to_frame.html").get_uri());
		}));
		js_api.api_injected.disconnect(run_test_inject_to_frame_on_api_injected);
		js_api.load_finished.disconnect(run_test_inject_to_frame);
	}
	
	private void run_test_inject_to_frame()
	{
		try
		{
			// Run JavaScript test code
			js_api.main_bridge.execute_script_from_file(File.new_for_path(TOP + JS_TESTS + "inject_to_frame.js"));
		}
		catch(JSError e)
		{
			assert_cmpstr("", Operator.EQ, e.message);
		}
	}
	
	private void run_test_inject_to_frame_on_api_injected(FrameBridge bridge)
	{
		if (bridge.frame_id == "iframe2")
		{
			// It is necessary to run test code after loading is finished
			bridge.load_finished.connect(run_test_inject_to_frame_on_load_finished);
		}
		
	}
	
	private void run_test_inject_to_frame_on_load_finished(FrameBridge bridge)
	{
		try
		{
			// Run JavaScript test code
			bridge.execute_script_from_file(File.new_for_path(TOP + JS_TESTS + "inject_to_frame_2.js"));
		}
		catch(JSError e)
		{
			assert_cmpstr("", Operator.EQ, e.message);
		}
	}
	
	private void on_song_changed(string? song, string? artist, string? album, string? art, string? state)
	{
		/* 
		 * Purpose of this callback is to test whether integration.js has been executed in
		 * a particular frame. integration.js passes Nuvola.frame_id as song argument.
		 */
		switch (song)
		{
		case "iframe1.html":
			assert_cmpstr(artist, Operator.EQ, "thefirst");
			break;
		case "iframe2.html":
			assert_cmpstr(artist, Operator.EQ, "iframe2");
			break;
		case "iframe3.html":
			assert_cmpstr(artist, Operator.EQ, "iframe3");
			break;
		case "inject_to_frame.html":
			assert_cmpstr(artist, Operator.EQ, "__my_frame__");
			break;
		default:
			assert_not_reached();
		}
		
		counter--;
		if (counter == 0)
			async_test_done();
	}
	
	private bool console_handler(string message, int line_number, string source_id)
	{
		console_messages += message;
		GLib.Test.message(message);
		return true;
	}
	
	public signal void async_test_done();
}
