using FsoFramework.Test;

public class Nuvola.ServicesManagerTest: FsoFramework.Test.TestCase
{
	private const string SERVICE_ID = "test_service";
	private Diorite.Storage? storage = null;
	private ServicesManager? manager = null;

	public ServicesManagerTest(){
		base("ServicesManagerTest");
		add_test("load_service_from_dir", test_load_service_from_dir);
		add_test("list_services_user", test_list_services_user);
		add_test("list_services_system", test_list_services_system);
		add_test("list_services_versioned", test_list_services_versioned);
		add_test("install_service", test_install_service);
		add_test("remove_service", test_remove_service);
	}
	
	public override void set_up(){
		storage = get_test_storage(Config.APPNAME + "/services");
		manager = new ServicesManager(storage);
	}
	
	public override void tear_down(){
		storage = null;
		manager = null;
		purge_test_dir();
	}
	
	private bool is_invalid_service(File service_dir, string reason){
		
		try{
			manager.load_service_from_dir(service_dir);
			Test.message("Service should be invalid, because %s, but no error thrown.", reason);
			return false;
		}
		catch(ServiceError e){
			Test.message(@"Caught error: #$(e.code): %s", e.message);
			return true;
		}
	}
	
	public void test_load_service_from_dir(){
		var service_dir = create_test_dir("service").get_child(SERVICE_ID);
		var metadata_file = service_dir.get_child("metadata.conf");
		var metadata = new Diorite.MapFile(metadata_file, false);
		
		/* Loading of service from non-existent directory must fail. */
		assert(is_invalid_service(service_dir, "doesn't exist"));
		
		/* Copy test service. */
		string spawn_stdout = null;
		string spawn_stderr = null;
		int result = 0;
		try{
			string[] args = {"cp", "-rf", TOP + "/tests/data/services/" + SERVICE_ID, service_dir.get_path()};
			Process.spawn_sync(null, args, null,  SpawnFlags.SEARCH_PATH, null, out spawn_stdout, out spawn_stderr, out result);
		}
		catch(SpawnError e){
			Test.message("Failed to copy service: %s", spawn_stderr ?? "null");
			Test.fail();
		}
		
		/* Reload metadata. */
		metadata = new Diorite.MapFile(metadata_file, false);
		
		/* Valid service must not fail. */
		try{
			var service = manager.load_service_from_dir(service_dir, false);
			assert(service.id != null);
			assert(service.name != null);
			assert(service.version > 0);
			assert(service.home_page != null);
			assert(service.version_minor >= 0);
			assert(service.maintainer_name != null);
			assert(service.maintainer_link != null);
			assert(service.api_major > 0);
			assert(service.api_minor >= 0);
			assert(service.resources != null);
			assert(service.config_dir != null);
		}
		catch(ServiceError e){
			Test.message(@"Loading of valid service failed: #$(e.code): %s", e.message);
			Test.fail();
		}
		
		try{
			
			/* Test check of mandatory fields */
			string[] mandatory_fields = {"name", "version", "home_page", "sandbox_pattern", "maintainer_name", "maintainer_link"};
			foreach(string k in mandatory_fields){
				string val = metadata[k];
				metadata[k] = "";
				metadata.save();
				assert(is_invalid_service(service_dir, k + " field missing"));
				metadata[k] = val ?? "";
			}
			
			/* Test validity check */
			var key = "sandbox_pattern";
			var value = metadata[key];
			metadata[key] = "foo***";
			metadata.save();
			assert(is_invalid_service(service_dir, key + " field is invalid"));
			metadata[key] = value ?? "";
			
			key = "version";
			value = metadata[key];
			metadata[key] = "0";
			metadata.save();
			assert(is_invalid_service(service_dir, key + " field is invalid"));
			metadata[key] = value ?? "";
			
			key = "version_minor";
			value = metadata[key];
			metadata[key] = "-1";
			metadata.save();
			assert(is_invalid_service(service_dir, key + " field is invalid"));
			metadata[key] = value ?? "";
			
			key = "api_major";
			value = metadata[key];
			metadata[key] = "0";
			metadata.save();
			assert(is_invalid_service(service_dir, key + " field is invalid"));
			metadata[key] = value ?? "";
			
			key = "api_minor";
			value = metadata[key];
			metadata[key] = "-1";
			metadata.save();
			assert(is_invalid_service(service_dir, key + " field is invalid"));
			metadata[key] = value ?? "";
			
			key = "api_major";
			value = metadata[key];
			metadata[key] = "10";
			metadata.save();
			assert(is_invalid_service(service_dir, key + " field is invalid"));
			metadata[key] = value ?? "";
			
			key = "api_minor";
			value = metadata[key];
			metadata[key] = "999";
			metadata.save();
			assert(is_invalid_service(service_dir, key + " field is invalid"));
			metadata[key] = value ?? "";
		}
		catch(Error e){
			Test.message("Unexpected error: %s", e.message);
			Test.fail();
		}
	}

	public void test_install_service(){
		Test.message("test_install_service");
		
		try{
			manager.install_service(File.new_for_path(TOP + "/tests/data/service-package-test.tar.gz"));
			assert(storage.user_data_dir.get_child("test").query_exists());
			Test.message("Installation successfull");
		}
		catch(ServiceError e){
			Test.message("Installation failed: %s".printf(e.message));
			Test.fail();
		}
		
		try{
			manager.install_service(File.new_for_path(TOP + "/tests/data/service-package-test.tar.gz"));
			assert(storage.user_data_dir.get_child("test").query_exists());
			Test.message("Second nstallation successfull");
		}
		catch(ServiceError e){
			Test.message("Second installation failed: %s".printf(e.message));
			Test.fail();
		}
		
		purge_test_dir();
		var test_service_dir = create_test_dir(DATA_USER).get_child("nuvolaplayer/services/test");
		
		string name;
		/*
		FIXME: move this check to JSApiTest
		try{
			name = "service-package-incomplete";
			manager.install_service(File.new_for_path("%s/tests/data/%s.tar.gz".printf(TOP, name)));
			Test.message("Installation of wrong package not failed: %s".printf(name));
			Test.fail();
		}
		catch(ServiceError e){
			Test.message(@"Caught error for %s: #$(e.code): %s", name, e.message);
			assert(!test_service_dir.query_exists());
		}
		*/
		
		try{
			name = "service-package-no-service_id";
			manager.install_service(File.new_for_path("%s/tests/data/%s.tar.gz".printf(TOP, name)));
			Test.message("Installation of wrong package not failed: %s".printf(name));
			Test.fail();
		}
		catch(ServiceError e){
			Test.message(@"Caught error for %s: #$(e.code): %s", name, e.message);
			assert(!test_service_dir.query_exists());
		}
		
		try{
			name = "service-package-fomat-1";
			manager.install_service(File.new_for_path("%s/tests/data/%s.tar.gz".printf(TOP, name)));
			Test.message("Installation of wrong package not failed: %s".printf(name));
			Test.fail();
		}
		catch(ServiceError e){
			Test.message(@"Caught error for %s: #$(e.code): %s", name, e.message);
			assert(!test_service_dir.query_exists());
		}
		
		try{
			name = "service-package-no-control";
			manager.install_service(File.new_for_path("%s/tests/data/%s.tar.gz".printf(TOP, name)));
			Test.message("Installation of wrong package not failed: %s".printf(name));
			Test.fail();
		}
		catch(ServiceError e){
			Test.message(@"Caught error for %s: #$(e.code): %s", name, e.message);
			assert(!test_service_dir.query_exists());
		}
		
		try{
			name = "service-package-test.tar.xz";
			manager.install_service(File.new_for_path("%s/tests/data/%s".printf(TOP, name)));
			Test.message("Installation of wrong package not failed: %s".printf(name));
			Test.fail();
		}
		catch(ServiceError e){
			Test.message(@"Caught error for %s: #$(e.code): %s", name, e.message);
			assert(!test_service_dir.query_exists());
		}
	}
	
	public void test_remove_service(){
		Test.log_set_fatal_handler(() => {return false;});
		string spawn_stdout = null;
		string spawn_stderr = null;
		int result = 0;
		
		var user_dir = create_test_dir(DATA_USER + "/nuvolaplayer");
		
		try{
			string[] args = {"cp", "-rvf", TOP + "/data/nuvolaplayer/services", user_dir.get_path()};
			Process.spawn_sync(null, args, null,  SpawnFlags.SEARCH_PATH, null, out spawn_stdout, out spawn_stderr, out result);
		}
		catch(SpawnError e){
			Test.message("Failed to install services: %s", spawn_stderr ?? "null");
			Test.fail();
		}
		if(result != 0){
			Test.message("Failed to install services: %s", spawn_stderr ?? "null");
			Test.fail();
		}
		
		var services = manager.list_services();
		var keys = services.get_keys();
		keys.sort(strcmp);
		foreach(var key in keys){
			var service = services.lookup(key);
			assert(service.removable);
			
			var service_dir = user_dir.get_child("services/" + service.id);
			assert(service_dir.query_exists());
			
			try{
				manager.remove_service(service);
				assert(!service_dir.query_exists());
				Test.message("Removal of %s successfull", service.name);
			}
			catch(ServiceError e){
				Test.message("Removal of %s failed: %s".printf(service.name, e.message));
				Test.fail();
			}
		}
	}
	
	public void test_list_services_user(){
		var user_dir = create_test_dir(DATA_USER + "/nuvolaplayer");
		var expected_services = copy_services(user_dir);
		var services = manager.list_services();
		assert_cmpuint(services.size(), Operator.EQ, expected_services.length());
		
		var keys = services.get_keys();
		keys.sort(strcmp);
		int i = 0;
		foreach(var key in keys){
			var service = services.lookup(key);
			assert_cmpstr(service.id, Operator.EQ, expected_services.nth_data(i));
			i++;
		}
	}
	
	public void test_list_services_system(){
		var user_dir = create_test_dir(DATA_DIR_1 + "/nuvolaplayer");
		var expected_services = copy_services(user_dir);
		var services = manager.list_services();
		assert_cmpuint(services.size(), Operator.EQ, expected_services.length());
		
		var keys = services.get_keys();
		keys.sort(strcmp);
		int i = 0;
		foreach(var key in keys){
			var service = services.lookup(key);
			assert_cmpstr(service.id, Operator.EQ, expected_services.nth_data(i));
			i++;
		}
	}
	
	private SList<string> copy_services(GLib.File dir){
		string spawn_stdout = null;
		string spawn_stderr = null;
		int result = 0;
		try{
			string[] args = {"cp", "-rvf", TOP + "/data/nuvolaplayer/services", dir.get_path()};
			Process.spawn_sync(null, args, null,  SpawnFlags.SEARCH_PATH, null, out spawn_stdout, out spawn_stderr, out result);
		}
		catch(SpawnError e){
			Test.message("Failed to install services: %s", spawn_stderr ?? "null");
			Test.fail();
		}
		
		if(result != 0){
			Test.message("Failed to install services: %s", spawn_stderr ?? "null");
			Test.fail();
		}
		
		FileInfo file_info;
		var expected_services = new SList<string>();
		try{
			var enumerator = dir.get_child("services").enumerate_children(
				FileAttribute.STANDARD_NAME, 0);
			while((file_info = enumerator.next_file()) != null){
				expected_services.prepend(file_info.get_name());
			}
		}
		catch(Error e){
			assert_not_reached();
		}
		
		expected_services.sort(strcmp);
		return expected_services;
	}
	
	/**
	 * Tests proper selection of service when there are multiple services with same id in diferent
	 * directories. Rules:
	 * 
	 * 1) higher version wins
	 * 2) service in user directory wins
	 * 3) service in the former data directory wins
	 */
	public void test_list_services_versioned(){
		string spawn_stdout = null;
		string spawn_stderr = null;
		int result;
		var user_dir = create_test_dir(DATA_USER + "/nuvolaplayer/services");
		var data_dir_1 = create_test_dir(DATA_DIR_1 + "/nuvolaplayer/services");
		var data_dir_2 = create_test_dir(DATA_DIR_2 + "/nuvolaplayer/services");
		
		File[] dirs = {user_dir, data_dir_1, data_dir_2};
		
		foreach(var dir in dirs){
			result = 0;
			try{
				string[] args = {"cp", "-rvf", TOP + "/tests/data/services/test_service", dir.get_path()};
				Process.spawn_sync(null, args, null,  SpawnFlags.SEARCH_PATH, null, out spawn_stdout, out spawn_stderr, out result);
			}
			catch(SpawnError e){
				Test.message("Failed to install services: %s", spawn_stderr ?? "null");
				Test.fail();
			}
			if(result != 0){
				Test.message("Failed to install services: %s", spawn_stderr ?? "null");
				Test.fail();
			}
		}
		
		var user_dir_service = user_dir.get_child("test_service");
		var user_dir_metadata = new Diorite.MapFile(user_dir_service.get_child("metadata.conf"), false);
		var data_dir_1_service = data_dir_1.get_child("test_service");
		var data_dir_1_metadata = new Diorite.MapFile(data_dir_1_service.get_child("metadata.conf"), false);
		var data_dir_2_service = data_dir_2.get_child("test_service");
		var data_dir_2_metadata = new Diorite.MapFile(data_dir_2_service.get_child("metadata.conf"), false);
		
		/* 2.0; 2.0; 2.0; all services have same version, user_dir_service wins */
		assert_test_service(user_dir_service, 2);
		
		/* 3.0; 2.0; 2.0; user_dir_service wins */
		set_version(user_dir_metadata, 3);
		assert_test_service(user_dir_service, 3);
		
		/* 3.0; 3.0; 2.0; user_dir_service wins */
		set_version(data_dir_1_metadata, 3);
		assert_test_service(user_dir_service, 3);
		
		/* 3.0; 3.0; 3.1; data_dir_2_service wins */
		set_version(data_dir_2_metadata, 3, 1);
		assert_test_service(data_dir_2_service, 3, 1);
		
		/* 3.0; 3.0; 4.0; data_dir_2_service wins */
		set_version(data_dir_2_metadata, 4, 0);
		assert_test_service(data_dir_2_service, 4, 0);
		
		/* 3.0; 4.0; 4.0; data_dir_1_service wins */
		set_version(data_dir_1_metadata, 4, 0);
		assert_test_service(data_dir_1_service, 4, 0);
		
		/* 3.0; 4.1; 4.0; data_dir_1_service wins */
		set_version(data_dir_1_metadata, 4, 1);
		assert_test_service(data_dir_1_service, 4, 1);
		
		/* 4.1; 4.1; 4.0; user_dir_service wins */
		set_version(user_dir_metadata, 4, 1);
		assert_test_service(user_dir_service, 4, 1);
	}
	
	private void set_version(Diorite.MapFile metadata, int version_major, int version_minor=0){
		metadata.set_int("version", version_major);
		metadata.set_int("version_minor", version_minor);
		try{
			metadata.save();
		}
		catch(Error e){
			assert_not_reached();
		}
	}
	
	private void assert_test_service(File dir, int version_major, int version_minor=0){
		var services = manager.list_services();
		assert_cmpuint(services.size(), Operator.EQ, 1);
		
		var service = services.lookup("test_service");
		assert(service != null);
		assert_cmpint(service.version, Operator.EQ, version_major);
		assert_cmpint(service.version_minor, Operator.EQ, version_minor);
		assert_cmpstr(service.resources.get_path(), Operator.EQ, dir.get_path());
	}
}
