#!/usr/bin/python2
# encoding: utf-8
#
# Copyright 2011-2012 Jiří Janoušek <janousek.jiri@gmail.com>
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are met: 
# 
# 1. Redistributions of source code must retain the above copyright notice, this
#    list of conditions and the following disclaimer. 
# 2. Redistributions in binary form must reproduce the above copyright notice,
#    this list of conditions and the following disclaimer in the documentation
#    and/or other materials provided with the distribution. 
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
# ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
# ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

# Extra functions for wscript file

import os
import sys
import subprocess
from waflib.Configure import conf
from waflib.Errors import ConfigurationError 

class Flags(object):
	"""Class that manage extra configure flags."""
	def __init__(self):
		self.flags = []
		self.flags_with_requires = []
		self.deps = {}
	
	def _requires_string(self, requires):
		result = []
		for key, value in requires.iteritems():
			result.append(self._dep_key(key, value))
			result.append(", ")
		result = result[:-1]
		if len(result) >= 3:
			result[-2] = " and "
		return "".join(result)
	
	def add(self, id, name, default, requires=None, vala_def=None):
		"""Adds a new flag."""
		flag = (id, name, default, requires, vala_def)
		self.flags.append(flag)
		if requires:
			self.flags_with_requires.append(flag)
			self._add_dep(flag)
	
	def _dep_key(self, name, version):
		if version:
			return "%s >= %s" % (name, version)
		else:
			return name
	
	def _add_dep(self, flag):
		id, name, default, requires, vala_def = flag
		for key, value in requires.iteritems():
			key = self._dep_key(key, value)
			try:
				self.deps[key].append(flag)
			except KeyError, e:
				self.deps[key] = [flag]
	
	def add_options(self, ctx):
		"""Adds flags to the options context."""
		for id, name, default, requires, vala_def in self.flags:
			if default:
				help_true = "Build with %s (default)." % name
				help_false = "Build without %s." % name
			else:
				help_true = "Build with %s." % name
				help_false = "Build without %s (default)." % name
			if requires:
					help_true += " Requires %s." % self._requires_string(requires)
			ctx.add_option("--" + id.replace("_", "-"), action='store_true', dest=id, help=help_true, default=default)
			ctx.add_option("--no-" + id.replace("_", "-"), action='store_false', dest=id, help=help_false)
	
	def store_result(self, ctx):
		"""Stores status of a feature to ctx.env.feature_<id> and appends Vala definitions"""
		for id, name, default, requires, vala_def in self.flags:
			if getattr(ctx.options, id):
				if vala_def is True:
					ctx.vala_def(id.upper())
				elif vala_def:
					ctx.vala_def(vala_def)
				ctx.env["feature_" + id] = True
			else:
				ctx.env["feature_" + id] = False
	
	def print_restrictive(self, ctx, message):
		"""Prints message and list of flags with non-empty 'requires' field"""
		if self.flags_with_requires:
			sys.stderr.write("\n %s\n" % message)
		for id, name, default, requires, vala_def in self.flags_with_requires:
			if getattr(ctx.options, id):
				sys.stderr.write(" * Feature %s requires %s.\n   You can disable it by a flag --no-%s.\n" % (name, self._requires_string(requires), id.replace("_", "-")))
	
	def get_feature_for_dep(self, ctx, dep_name, dep_version=None):
		"""Returns information about features that require particular dependency."""
		flags = self.deps.get(self._dep_key(dep_name, dep_version), None)
		if(flags):
			for id, name, default, requires, vala_def in flags:
				if getattr(ctx.options, id):
					yield "Feature %s requires %s.\n   You can disable it by a flag --no-%s." % (name, self._requires_string(requires), id.replace("_", "-"))
	
	def print_info(self):
		"""Prints features and their flags"""
		print "Use `./waf configure --feature` to turn the feature on."
		print "Use `./waf configure --no-feature` to turn the feature off."
		print
		print "Available features: \n"
		for id, name, default, requires, vala_def in self.flags:
			if default:
				help_true = "Build with %s (default)." % name
				help_false = "Build without %s." % name
			else:
				help_true = "Build with %s." % name
				help_false = "Build without %s (default)." % name
			if requires:
					help_true += " Requires %s." % self._requires_string(requires)
			print "--" + id.replace("_", "-")
			print "\t" + help_true
			print "--no-" + id.replace("_", "-")
			print "\t" + help_false
	
	def print_summary(self, ctx):
		"""Prints status of features, returns list of enabled and disabled features"""
		enabled = []
		disabled = []
		sys.stderr.write("\nFeatures: \n")
		for id, name, default, requires, vala_def in self.flags:
			if getattr(ctx.options, id):
				ctx.msg(name, "ON", "GREEN")
				enabled.append(name);
			else:
				ctx.msg(name, "OFF", "RED")
				disabled.append(name)
		return ", ".join(enabled), ", ".join(disabled)

flags = Flags()

@conf
def vala_def(ctx, vala_definition):
	"""Appends a Vala definition"""
	if not hasattr(ctx.env, "VALA_DEFINES"):
		ctx.env.VALA_DEFINES = []
	if isinstance(vala_def, tuple) or isinstance(vala_def, list):
		for d in vala_definition:
			ctx.env.VALA_DEFINES.append(d)
	else:
		ctx.env.VALA_DEFINES.append(vala_definition)

@conf
def check_dep(ctx, pkg, uselib, version, mandatory=True, store=None, vala_def=None, define=None):
	"""Wrapper for ctx.check_cfg that stores errors instead of termination of the script."""
	result = True
	try:
		ctx.check_cfg(package=pkg, uselib_store=uselib, atleast_version=version, mandatory=True, args = '--cflags --libs')
		if vala_def:
			ctx.vala_def(vala_def)
		if define:
			for key, value in define.iteritems():
				ctx.define(key, value)
	except ConfigurationError, e:
		if mandatory:
			fail = "Unable to satisfy dependency %s >= %s" % (pkg, version)
			ctx.append_error(fail)
			for fail in flags.get_feature_for_dep(ctx, pkg, version):
				ctx.append_error(fail)
		result = False
	if store is not None:
		ctx.env[store] = result
	return result

@conf
def check_prog(ctx, name, mandatory=True):
	"""Wrapper for ctx.find_program that stores errors instead of termination of the script."""
	try:
		ctx.find_program(name, mandatory=True)
		return True
	except ConfigurationError:
		if mandatory:
			ctx.append_error("Unable to find program '%s'" % name)
			for fail in flags.get_feature_for_dep(ctx, name):
				ctx.append_error(fail)
		return False

@conf
def append_error(ctx, fail):
	"""Appends error message"""
	try:
		ctx.conf_errors.append(fail)
	except AttributeError:
		ctx.conf_errors = [fail]

@conf
def check_status(ctx, fail_message=""):
	"""Checks status of configuration. Lists errors and terminates script if there are any errors."""
	try:
		errors = ctx.conf_errors
	except AttributeError, e:
		errors = None
	if errors:
		ctx.msg("Configuration status", "FAIL", "RED")
		
		for error in errors:
			sys.stderr.write(" * %s\n" % error)
		sys.stderr.write("\n")
		ctx.fatal("Configuration failed. Please read file INSTALL for details about dependencies." + fail_message)
	else:
		sys.stderr.write("\n")
		ctx.msg("Configuration status", "OK", "GREEN")




def parse_version(version, stage):
	"""
	Parses version string and returns
	VERSION, SERIES, MAJOR, MINOR, BUGFIX, SUFFIX, DEV_STAGE, REVISION, REVISION_ID
	"""
	import re
	m = re.match("^(\d+)\.(\d+)(?:\.(\d+))?(?:\~(unstable|beta\d+))?$", version)
	if not m:
		raise ValueError("VERSION '%s' doesn't match regular expression" % version)
	major = int(m.group(1) or 0)
	minor = int(m.group(2) or 0)
	bugfix = int(m.group(3) or 0)
	series = "%s.%s" % (major, minor)
	
	try:
		try:
			num, id = open("revision-info", "r").read().split(" ")
		except Exception, e:
			output = subprocess.Popen(["bzr", "revision-info"], stdout=subprocess.PIPE).communicate()[0]
			num, id = output.split(" ")
	except Exception, e:
		revision = 0
		revision_id = "UNKNOWN_REVISION_ID"
	else:
		revision = str(int(num))
		revision_id = str(id).strip()
	
	m = re.match("^(stable|unstable|beta\d+)$", stage)
	if not m:
		raise ValueError("Stage '%s' doesn't match regular expression" % stage)
	
	if stage == "stable":
		dev_stage = stage
		suffix = ""
	else:
		if "beta" in stage:
			dev_stage = "beta"
		else:
			dev_stage = "unstable"
		suffix = "~r%s.%s" % (revision, stage)
		version += suffix 

	return version, series, major, minor, bugfix, suffix, dev_stage, revision, revision_id

